#!/usr/bin/env node

const { parse } = require("@babel/parser");
const { template, traverse, types: t } = require("@babel/core");
const generate = require("@babel/generator").default;
const fs = require("fs");
const path = require("path");
const { Command, option } = require("commander");
const mp = require("../lib/mp-traverse");
const prettier = require("prettier");
const moment = require("moment");
const BeanboomTemplate = require("../lib/Beanboom/Template");
const ScriptStrategy = require("../lib/Beanboom/ScriptStrategy");
const AngularScript = require("../lib/Beanboom/AngularScript");
const ReactScript = require("../lib/Beanboom/ReactScript");
const CodeSnippet = require("../lib/Beanboom/CodeSnippet");
const package = JSON.parse(
    fs.readFileSync(path.resolve(__dirname, "../package.json"), {
        encoding: "utf-8",
    })
);
const { getFileAst, writeFileAst } = require("../lib/utils");

const program = new Command();

program.version(package.version);

program
    .command("create")
    .option("-a --action <action>", "动作")
    .option("-f --filePath <filePath>", "文件路径")
    .option("-c --className <className>", "类名称")
    .option("-m --methodName <methodName>", "方法名称")
    .option("-p --propertyName <propertyName>", "方法名称")
    .action(function (options) {
        if (!fs.existsSync(options.filePath)) {
            fs.writeFileSync(options.filePath, "", { encoding: "utf-8" });
        }
        let ast = getFileAst(options.filePath);

        traverse(ast, {
            ClassProperty(path) {
                let comments = path.node.leadingComments;
                if (comments) {
                    t.removeComments(path.node);
                    t.addComment(path.node, "leading", comments[0].value);
                }
            },
        });
        let codeSnippet = new CodeSnippet(ast);
        if (options.action === "method") {
            codeSnippet.setMethod(options.className, options.methodName);
        } else if (options.action === "property") {
            codeSnippet.setProperty(options.className, options.propertyName);
        } else if (options.action === "removeProperty") { 
            codeSnippet.removeProperty(options.className, options.propertyName);
        } else if (options.action === "class") {
            ast.program.body.push(codeSnippet.getClass(options.className));
        }
        writeFileAst(options.filePath, ast, "script", true);
    });

program
    .command("build")
    .option("-s, --source <source>", "来源目录")
    .option("-c, --component <component>", "组件名称")
    .option("-d, --directory <directory>", "来源目录（批量）")
    .option("-o, --output <output>", "输出目录")
    .option("-w, --watch", "监听目录")
    .option("-t, --target <target>", "输出类型")
    .option("-m, --module <module>", "模块名称")
    .action(function (options) {
        const SRC_PATH = path.resolve(options.source || options.directory);
        const DEST_PATH = path.resolve(options.output);
        const MODULE_PATH = options.module
            ? path.join(SRC_PATH, options.module)
            : SRC_PATH;
        const MODULE_CONFIG_PATH = path.join(MODULE_PATH, "Module.json");

        // 脚本转化
        let scriptStrategy = new ScriptStrategy();
        scriptStrategy.add("angular", function (script, moduleConfig) {
            return new AngularScript(script, moduleConfig).getCode();
        });
        scriptStrategy.add("react", function (script) {
            return new ReactScript(script).getCode();
        });

        // 模块配置
        let moduleConfig = {};
        if (fs.existsSync(path.join(MODULE_PATH, "Module.json"))) {
            moduleConfig = require(path.join(MODULE_PATH, "Module.json"));
        }

        const Config = {
            mp: {
                template: {
                    suffix: "wxml",
                },
                style: {
                    suffix: ".wxss",
                },
            },
            react: {
                template: {
                    suffix: "jsx",
                },
                style: {
                    // suffix: STYLE_PATH ? path.extname(STYLE_PATH) : "",
                },
            },
            angular: {
                template: {
                    suffix: "html",
                },
                style: {
                    // suffix: STYLE_PATH ? path.extname(STYLE_PATH) : "",
                },
            },
        }[options.target];

        function getFIlepaths(directoryPath, extension) {
            const items = fs.readdirSync(directoryPath);
            return items.filter((item) => item.indexOf(".html") !== -1);
        }

        function compile(src, dest, component) {
            // 输出文件
            if (options.output) {
                // 输入路径
                let stylePath;
                for (const suffix of ["css", "less", "sass"]) {
                    if (
                        fs.existsSync(
                            path.join(
                                src,
                                component + "/" + component + "." + suffix
                            )
                        )
                    ) {
                        stylePath = path.join(
                            src,
                            component + "/" + component + "." + suffix
                        );
                        break;
                    }
                }
                let paths = {
                    script: `${src}/${component}/${component}.js`,
                    template: getFIlepaths(`${src}/${component}/`).map(
                        (item) => `${src}/${component}/${item}`
                    ),
                    style: stylePath,
                    config: `${src}/${component}/${component}.json`,
                };

                // 输出路径
                let outputs = {
                    script: `${dest}/${component}/${component}.js`,
                    // template: `${dest}/${component}/${component}.${Config.template.suffix}` paths.template.map(item=>),
                    template: getFIlepaths(`${src}/${component}/`).map(
                        (item) =>
                            `${dest}/${component}/${item.replace(
                                "html",
                                Config.template.suffix
                            )}`
                    ),
                };
                paths["style"] &&
                    (outputs[
                        "style"
                    ] = `${dest}/${component}/${component}${path.extname(
                        paths["style"]
                    )}`);

                // 内容
                let contents = {};
                for (const key in paths) {
                    if (Object.hasOwnProperty.call(paths, key)) {
                        const element = paths[key];
                        if (Array.isArray(element)) {
                            contents[key] = element.map((item) =>
                                fs.readFileSync(item, "utf-8")
                            );
                        } else {
                            if (fs.existsSync(element)) {
                                if (key === "config") {
                                    contents["config"] = JSON.parse(
                                        fs.readFileSync(paths[key], "utf-8")
                                    );
                                } else {
                                    contents[key] = fs.readFileSync(
                                        paths[key],
                                        "utf-8"
                                    );
                                }
                            }
                        }
                    }
                }

                // 抽象节点树
                let asts = {};
                if (contents["script"]) {
                    asts["script"] = parse(contents["script"], {
                        sourceType: "module",
                        plugins: ["classProperties"],
                    });
                }
                if (contents["template"].length > 0) {
                    asts["template"] = contents["template"].map((item) =>
                        parse(item, {
                            plugins: ["jsx"],
                        })
                    );
                }

                // 输出目录（初始化）
                if (!fs.existsSync(`${dest}/${component}/`)) {
                    fs.mkdirSync(`${dest}/${component}/`);
                }

                // 代码编译
                let astOptions = {};
                if (options.target === "react") {
                    // astOptions.style = { suffix: path.extname(STYLE_PATH) };
                    // react.scriptTransform(ast, astOptions);
                    // react.viewTransform(viewAst);c
                    contents["template"] = "";
                    if (asts.script) {
                        contents["script"] = scriptStrategy.get("react")(
                            asts.script
                        );
                    }
                } else if (options.target === "angular") {
                    if (asts.template && asts.template.length > 0) {
                        contents["template"] = asts.template.map((item) =>
                            new BeanboomTemplate(item).getCode()
                        );
                    }
                    if (asts.script) {
                        contents["script"] = scriptStrategy.get("angular")(
                            asts.script,
                            moduleConfig
                        );
                    }
                } else if (options.target === "mp") {
                    mp.scriptTraverse(asts.script);
                    mp.viewTraverse(asts.template);
                    contents["script"] = prettier.format(
                        generate(asts.script).code,
                        {
                            parser: "babel",
                            tabWidth: 4,
                        }
                    );
                    contents["template"] = generate(asts.template).code.replace(
                        /;$/,
                        ""
                    );
                    fs.writeFileSync(
                        `${dest}/${component}/${component}.json`,
                        `{"component": true}`,
                        {
                            encoding: "utf-8",
                        }
                    );
                }

                // 输出代码
                for (const key in outputs) {
                    if (key === "template") {
                        outputs[key].forEach(function (item) {
                            if (!fs.existsSync(path.dirname(item))) {
                                fs.mkdirSync(path.dirname(item));
                            }
                        });
                    } else {
                        if (!fs.existsSync(path.dirname(outputs[key]))) {
                            fs.mkdirSync(path.dirname(outputs[key]));
                        }
                    }

                    if (key === "template") {
                        outputs[key].forEach(function (path, index) {
                            fs.writeFileSync(path, contents[key][index], {
                                encoding: "utf-8",
                            });
                        });
                    } else {
                        fs.writeFileSync(outputs[key], contents[key], {
                            encoding: "utf-8",
                        });
                    }
                }
            }
        }

        let sourcePath = options.module
            ? path.resolve(options.source, options.module)
            : path.resolve(options.source);
        let outputPath = options.module
            ? path.resolve(options.output, options.module)
            : path.resolve(options.output);
        if (options.watch) {
            fs.watch(
                options.directory || options.source,
                {
                    recursive: true,
                },
                (eventType, filename) => {
                    console.log(
                        `[${moment().format("HH:mm:ss")}]`,
                        eventType,
                        filename,
                        path.dirname(filename).split(path.sep).pop()
                    );
                    compile(
                        sourcePath,
                        outputPath,
                        path.dirname(filename).split(path.sep).pop()
                    );
                }
            );
        } else {
            if (options.component) {
                compile(sourcePath, outputPath, options.component);
            } else {
                let items = fs.readdirSync(sourcePath);
                for (const item of items) {
                    if (item !== "Module.json") {
                        compile(sourcePath, outputPath, item);
                    }
                }
            }
        }
    });

program.parse(process.argv);
