const { types, traverse } = require("@babel/core");
const generate = require("@babel/generator").default;
const template = require("@babel/template").default;
const { parse } = require("@babel/parser");
const t = require("@babel/types");
const ObjectExpression = require("../types/ObjectExpression");
const utils = require("../utils");

class Script {
    /**
     * 模块名称
     */
    moduleName = null;

    /**
     * 指令名称
     */
    directiveName = null;

    /**
     * 指令选项
     */
    options = new ObjectExpression();

    /**
     * 状态
     */
    state = null;

    /**
     * 属性
     */
    props = null;

    /**
     * 控制器方法的身体部分
     */
    controllerBody = [];

    /**
     * 构造方法的内容
     */
    linkBody = [];

    /**
     * 导入
     */
    imports = [];

    /**
     * 依赖模块
     */
    useModules = [];

    /**
     * 构造方法
     */
    constructor(ast) {
        this.ast = ast;
    }

    getAst() {
        let useModules = "";
        let useModules4QuotationMarks = "";
        let func = "";
        if (this.useModules && this.useModules.elements) {
            useModules = this.useModules.elements
                .map(function (item) {
                    return item.value;
                })
                .join(", ");
            useModules4QuotationMarks = this.useModules.elements
                .map(function (item) {
                    return `"${item.value}"`;
                })
                .join(", ");
                func = `[${useModules4QuotationMarks}, function(${useModules}) {
                    return OPTIONS;
                }]`
        } else {
            func = `function() {
                return OPTIONS;
            }`
        }
        let path = template(
            `angular.module(MODULE_NAME).directive(DIRECTIVE_NAME, ${func})`
        )({
            MODULE_NAME: this.moduleName,
            DIRECTIVE_NAME: this.directiveName,
            OPTIONS: this.options.getNode(),
        });
        let ast = parse(generate(path).code, {
            sourceType: "module",
        });
        if (this.imports.length > 0) {
            this.imports.forEach((item) => {
                ast.program.body.unshift(item.node);
            });
        }
        traverse(ast, {
            ThisExpression(path) {
                let parent = path.findParent((path) =>
                    path.isMemberExpression()
                );
                if (parent.node.property.name === "getContext") {
                    return;
                }
                path.replaceWith(types.identifier("$scope"));
            },
            MemberExpression(path) {
                if (
                    types.isThisExpression(path.node.object) &&
                    types.isIdentifier(path.node.property, { name: "props" })
                ) {
                    path.replaceWith(types.identifier("$scope"));
                }
            },
        });
        return ast;
    }

    setImport(path) {
        this.imports.push(path);
    }

    setModuleName(name) {
        this.moduleName = t.stringLiteral(name);
    }

    setUseModules(modules) {
        this.useModules = modules;
    }

    setDirectiveName(name) {
        name = utils.toCamelCase(name);
        this.directiveName = t.stringLiteral(name);
    }

    setContext() {
        this.controllerBody.push(
            template(`this.getContext = function(){ return $scope}`)()
        );
    }

    setRestrict(value) {
        this.options.setProperty("restrict", value || types.stringLiteral("E"));
    }

    setReplace(value) {
        this.options.setProperty(
            "replace",
            value || types.booleanLiteral(true)
        );
    }

    setTransclude(value) {
        this.options.setProperty(
            "transclude",
            value || types.booleanLiteral(true)
        );
    }

    setController(params = [], body = []) {
        let p = ["$scope"].concat(params);
        if (this.controllerBody.length > 0) {
            this.options.setMethod("controller", p, this.controllerBody);
        }
    }

    getController() {
        return this.options;
    }

    setLink(params = ["$scope"], body = []) {
        body = this.linkBody.concat(body);
        if (body.length > 0) {
            this.options.setMethod("link", params, body);
        }
    }

    setScope(value = []) {
        let scopes = JSON.parse(JSON.stringify(value));
        scopes.properties.map((element) => {
            if (types.isIdentifier(element.value)) {
                switch (element.value.name) {
                    case "String":
                    case "Number":
                    case "Boolean":
                        element.value = types.stringLiteral("@");
                        break;
                    case "Function":
                        element.value = types.stringLiteral("&");
                        break;
                    case "Array":
                    case "Object":
                        element.value = types.stringLiteral("=");
                        break;
                }
            } else {
                switch (element.value.property.name) {
                    case "string":
                        element.value = types.stringLiteral("@");
                        break;
                    case "function":
                        element.value = types.stringLiteral("&");
                        break;
                    case "array":
                    case "number":
                    case "boolean":
                    case "object":
                        element.value = types.stringLiteral("=");
                        break;
                }
            }

            return element;
        });
        this.options.setProperty("scope", scopes);
    }

    setDefaultProps(value) {
        this.controllerBody.push(
            template(`
            $scope.handleProperties = function() {
                let props = VALUE;
                Object.keys(props).forEach(function(key) {
                    Object.defineProperty($scope, key, {
                        get: function() {
                            return this[key] || props[key]
                        }
                    })
                })
            }
        `)({ VALUE: value })
        );
        this.linkBody.push(template("$scope.handleProperties()")());
    }

    setState(value, comments) {
        let expressionStatement = types.expressionStatement(
            types.assignmentExpression(
                "=",
                types.memberExpression(
                    types.identifier("$scope"),
                    types.identifier("state")
                ),
                value
            )
        );
        expressionStatement.leadingComments = comments;
        this.controllerBody.push(expressionStatement);
    }

    setProperty(name, value) {
        let expressionStatement = types.expressionStatement(
            types.assignmentExpression(
                "=",
                types.memberExpression(types.identifier("$scope"), name),
                value
            )
        );
        this.controllerBody.push(expressionStatement);
    }

    setWatch(value) {
        this.controllerBody.push(
            template(`$scope.watch = VALUE`)({ VALUE: value })
        );
        this.linkBody.push(
            template(`for (const key in $scope.watch) {
                $scope.$watch(key, $scope.watch[key], true)
            }`)()
        );
    }

    setRequire(value) {
        this.options.setProperty("require", value);
    }

    setTemplate(value) {
        this.options.setProperty("template", value);
    }

    setControllerAs(value) {
        this.options.setProperty("controllerAs", value);
    }

    setMethod(name, params = [], body = []) {
        let expressionStatement = types.expressionStatement(
            types.assignmentExpression(
                "=",
                types.memberExpression(types.identifier("$scope"), name),
                types.functionExpression(null, params, body)
            )
        );
        this.controllerBody.push(expressionStatement);
    }
}

module.exports = Script;
