const template = require("@babel/template").default;
const { traverse, types } = require("@babel/core");
const JSXExpressionContainer = require("../types/JSXExpressionContainer");
const generate = require("@babel/generator").default;
const { toKebabCase } = require("../utils");

class Template {
    constructor(ast) {
        this.ast = ast;
    }

    getStyle(value) {
        value = generate(value, {
            minified: true,
        })
            .code.replace(/\"/g, "'")
            .replace(/\{\{(.*?)\}\}/, "{$1}");
        return types.jsxAttribute(
            types.jsxIdentifier("ng-style"),
            types.stringLiteral(`${value}`)
        );
    }

    getVar(name) {
        return types.stringLiteral(`{{${name}}}`);
    }

    getIf(elementName, condition, attributes = [], children = []) {
        if (elementName === "input") {
            return types.jsxElement(
                types.jsxOpeningElement(
                    types.jsxIdentifier(elementName),
                    [
                        types.jsxAttribute(
                            types.jsxIdentifier("ng-if"),
                            types.stringLiteral(
                                generate(condition, {
                                    minified: true,
                                }).code.replace(/\"/g, "'")
                            )
                        ),
                        ...attributes,
                    ],
                    true
                ),
                null,
                []
            );
        } else {
            return types.jsxElement(
                types.jsxOpeningElement(types.jsxIdentifier(elementName), [
                    types.jsxAttribute(
                        types.jsxIdentifier("ng-if"),
                        types.stringLiteral(
                            generate(condition, {
                                minified: true,
                            }).code.replace(/\"/g, "'")
                        )
                    ),
                    ...attributes,
                ]),
                types.jsxClosingElement(types.jsxIdentifier(elementName)),
                children
            );
        }
    }

    getCodeBlocks(paths) {
        let codeblocks = [];
        paths.map(function (path) {
            codeblocks.push(
                types.jsxAttribute(
                    types.jsxIdentifier("ng-bind-html"),
                    types.stringLiteral(path)
                )
            );
        });
        return codeblocks;
    }

    getConditions(paths) {
        let conditions = [];
        paths.map(function (path) {
            let jsxElement = path.node.expression.right;
            conditions.push(
                types.jsxElement(
                    types.jsxOpeningElement(
                        types.jsxIdentifier(
                            jsxElement.openingElement.name.name
                        ),
                        [
                            types.jsxAttribute(
                                types.jsxIdentifier("ng-if"),
                                types.stringLiteral(
                                    generate(path.node.expression.left, {
                                        minified: true,
                                    }).code.replace(/\"/g, "'")
                                )
                            ),
                            ...jsxElement.openingElement.attributes,
                        ]
                    ),
                    types.jsxClosingElement(
                        types.jsxIdentifier(jsxElement.openingElement.name.name)
                    ),
                    jsxElement.children
                )
            );
        });
        return conditions;
    }

    /**
     * 获取属性
     * @deprecated
     */
    getAttributes(paths) {
        let attributes = [];
        paths.map(function (path) {
            if (types.isStringLiteral(path.node.value)) {
                attributes.push(
                    types.jsxAttribute(
                        types.jsxIdentifier(path.node.name.name),
                        path.node.value
                    )
                );
            } else if (types.isJSXExpressionContainer(path.node.value)) {
                attributes.push(
                    types.jsxAttribute(
                        types.jsxIdentifier("ng-" + path.node.name.name),
                        types.stringLiteral(
                            generate(path.node.value, {
                                minified: true,
                            }).code.replace(/\"/g, "'")
                        )
                    )
                );
            }
        });
        return attributes;
    }

    getAttribute(name, value, elementName) {
        let jsxIdentifier;
        if (
            (["checked", "value", "disabled"].includes(name) &&
                ["input", "button"].includes(elementName)) ||
            (["src"].includes(name) && ["img"].includes(elementName))
        ) {
            jsxIdentifier = types.jsxIdentifier("ng-" + name);
        } else {
            jsxIdentifier = types.jsxIdentifier(toKebabCase(name));
        }
        value = generate(value, {
            minified: true,
        }).code.replace(/\"/g, "'");
        return types.jsxAttribute(
            jsxIdentifier,
            types.stringLiteral(`{${value}}`)
        );
    }

    getLoop(name, item, items, attributes = [], children = [], key = "") {
        let left = "";
        if (key) {
            left = `(${key}, ${item})`;
        } else {
            left = item;
        }
        let keyAttr = attributes.find((item) => item.name.name === "key");
        if (keyAttr) {
            items += ` track by ${keyAttr.value.value}`;
            attributes = attributes.filter((item) => item.name.name !== "key");
        }
        return types.jsxElement(
            types.jsxOpeningElement(name, [
                types.jsxAttribute(
                    types.jsxIdentifier("ng-repeat"),
                    types.stringLiteral(`${left} in ${items}`)
                ),
                ...attributes,
            ]),
            types.jsxClosingElement(name),
            children
        );
    }

    setChildren(path) {
        path.replaceWith(
            types.expressionStatement(
                types.jsxElement(
                    types.jsxOpeningElement(
                        types.jsxIdentifier("ng-transclude"),
                        []
                    ),
                    types.jsxClosingElement(
                        types.jsxIdentifier("ng-transclude")
                    ),
                    []
                )
            )
        );
    }

    setExpressionContainer(paths) {
        for (let i = 0; i < paths.length; i++) {
            const path = paths[i];
            path.replaceWith();
        }
    }

    getClassName(value) {
        if (types.isStringLiteral(value)) {
            return types.jsxAttribute(types.jsxIdentifier("class"), value);
        } else if (types.isJSXExpressionContainer(value)) {
            return types.jsxAttribute(
                types.jsxIdentifier("ng-class"),
                types.stringLiteral(
                    generate(value.expression, {
                        minified: true,
                    })
                        .code.replace(/"/g, "'")
                        .replace(";", "")
                )
            );
        }
    }

    getChildren(path) {
        let parent = path.findParent((path) => path.isJSXElement());
        parent.node.openingElement.attributes.push(
            types.JSXAttribute(types.JSXIdentifier("ng-transclude"), null)
        );
        parent.node.children = [];
        // return types.jsxElement(
        //     types.jsxOpeningElement(types.jsxIdentifier("span"), [
        //         types.jsxAttribute(types.jsxIdentifier("ng-transclude")),
        //     ]),
        //     types.jsxClosingElement(types.jsxIdentifier("span")),
        //     []
        // );
    }

    getChildrens(quantity) {
        let paths = [];
        for (let i = 0; i < quantity; i++) {
            paths[i] = types.jsxElement(
                types.jsxOpeningElement(
                    types.jsxIdentifier("ng-transclude"),
                    []
                ),
                types.jsxClosingElement(types.jsxIdentifier("ng-transclude")),
                []
            );
        }
        return paths;
    }

    getEvent(eventName, elementName, functionName, functionParams = []) {
        const ast = {
            type: "Program",
            body: [].concat(
                types.expressionStatement(
                    types.callExpression(
                        types.identifier(functionName),
                        functionParams
                    )
                )
            ),
        };
        let jsxIdentifier;
        if (eventName === "onChange" && elementName !== "input") {
            jsxIdentifier = types.jsxIdentifier(toKebabCase(eventName));
        } else {
            jsxIdentifier = types.jsxIdentifier(
                "ng-" + eventName.replace("on", "").toLowerCase()
            );
        }
        return types.jsxAttribute(
            jsxIdentifier,
            types.stringLiteral(
                generate(ast, {
                    minified: true,
                })
                    .code.replace(/"/g, "'")
                    .replace("this,", "")
                    .replace(";", "")
            )
        );
    }
}

module.exports = Template;
