const ReactScript = require("../React/Script");
const AngularScript = require("../Angular/Script");
const AngularController = require("../Angular/Controller");
const { ast2code } = require("../utils");
const ObjectExpression = require("../types/ObjectExpression");
const ClassDeclaration = require("../types/ClassDeclaration");
const BaseScript = require("./Script");
const { traverse } = require("@babel/core");
const _ = require("underscore");

class Component extends BaseScript {
    constructor(ast, options = {}) {
        super(ast);
        this.ast = ast;
        this.options = options;
        this.reactScript = new ReactScript(ast);
        this.angularScript = new AngularScript();
        if (options.name) {
            this.angularScript.setModuleName(options.name);
        }
        this.transformWatch();
        // this.transformImport();
        this.transformUseModules();
        this.transformName();
        this.transformTemplate();
        this.transformRestrict();
        this.transformReplace();
        this.transformTransclude();
        this.transformScope();
        this.transformDefaultProperty();
        this.transformProps();
        this.transformState();
        this.trasnformRequire();
        this.transformMethod();
        this.transformProperty();
        this.transformController();
        this.transformConstructor();
    }

    trasnformRequire() {
        let property = this.reactScript.getProperty("require");
        if (property) {
            this.angularScript.setRequire(property.getValue());
            this.reactScript.removeProperty("require");
        }
    }

    transformWatch() {
        let method = this.reactScript.getComponentDidUpdate();
        if (method) {
            let obj = new ObjectExpression();
            for (const item of method.getBody().body) {
                obj.setMethod(
                    item.test.left.property.name,
                    ["newVal"],
                    item.consequent
                );
            }
            this.angularScript.setWatch(obj.getNode());
        }
    }

    transformUseModules() {
        let useModules = this.reactScript.getProperty("useModules");
        if (useModules) {
            this.angularScript.setUseModules(useModules.getValue());
        }
    }

    transformImport() {
        let imports = this.reactScript.getImport();
        if (Array.isArray(imports) && imports.length > 0) {
            imports.forEach((path) => {
                this.angularScript.setImport(path);
            });
        }
    }

    transformName() {
        let myClass = this.reactScript.getClassName();
        if (myClass) {
            let prefix =
                (this.options.component && this.options.component.prefix) || "";
            this.angularScript.setDirectiveName(prefix + myClass.getName());
        }
    }

    transformProperty() {
        let properties = this.reactScript.getProperties();
        if (Array.isArray(properties) && properties.length > 0) {
            properties.forEach((property) => {
                if (property.getName().name === "context") {
                    this.angularScript.setContext();
                } else if (property.getName().name !== "useModules") {
                    this.angularScript.setProperty(
                        property.getName(),
                        property.getValue()
                    );
                }
            });
        }
    }

    transformMethod() {
        let methods = this.reactScript.getMethods();
        if (methods) {
            methods.forEach((method) => {
                if (
                    !["render", "componentDidUpdate"].includes(
                        method.getName().name
                    )
                ) {
                    this.angularScript.setMethod(
                        method.getName(),
                        method.getParams(),
                        method.getBody()
                    );
                }
            });
        }
    }

    transformConstructor() {
        let method = this.reactScript.getMethod("constructor");
        let body = (method && method.getBody().body) || [];
        // console.log(method.getParams());
        let params = ((method && method.getParams()) || []).map(function (
            item
        ) {
            return item.name;
        });
        this.angularScript.setLink(
            ["$scope"].concat(params),
            // ["$scope", "$element", "$attrs", "$controllers", "$transclude"],
            body
        );
    }

    transformDefaultProperty() {
        let props = this.reactScript.getDefaultProps();
        if (props) {
            this.angularScript.setDefaultProps(props.getPath());
        }
    }

    transformProps() {
        let props = this.reactScript.getProps();
        if (props) {
            this.angularScript.setScope(props.getPath());
        }
    }

    transformController() {
        let params = [];
        if (this.reactScript.getIdentifier("$element")) {
            params = ["$element"];
        }
        if (this.reactScript.getIdentifier("$attrs")) {
            params = ["$element", "$attrs"];
        }
        this.angularScript.setController(params);
    }

    transformState() {
        let state = this.reactScript.getProperty("state");
        if (state) {
            this.angularScript.setState(
                state.getValue(),
                state.getLeadingComments()
            );
        }
    }

    transformScope() {
        let props = this.reactScript.getProperty("props");
        if (props) {
            this.angularScript.setScope(props.getValue());
        }
    }

    transformTransclude() {
        let transclude = this.reactScript.getProperty("transclude");
        this.angularScript.setTransclude(transclude && transclude.getValue());
    }

    transformReplace() {
        let replace = this.reactScript.getProperty("replace");
        this.angularScript.setReplace(replace && replace.getValue());
    }

    transformRestrict() {
        let restrict = this.reactScript.getProperty("restrict");
        this.angularScript.setRestrict(restrict && restrict.getValue());
    }

    transformTemplate() {
        let render = this.reactScript.getRender();
        if (render) {
            this.angularScript.setTemplate(render.getReturnValue());
        }
    }

    getCode() {
        let path = this.reactScript.getClass();
        this.reactScript.removePropType();
        if (path) {
            path.replaceWith(this.angularScript.getAst());
        }
        return ast2code(this.ast);
    }
}

class Controller {
    constructor(ast) {
        this.reactScript = new ReactScript(ast);
        this.angularScript = new AngularController();
        this.angularScript.setName(this.reactScript.getClass().getName());
        this.transformProperty();
        this.transformMethod();
        this.transformConstructor();
    }

    transformProperty() {
        let properties = this.reactScript.getProperties();
        if (!_.isEmpty(properties)) {
            properties.forEach((property) => {
                if (property.getName().name !== "useModules") {
                    this.angularScript.setProperty(
                        property.getName(),
                        property.getValue()
                    );
                }
            });
        }
    }

    transformMethod() {
        let methods = this.reactScript.getMethods();
        if (!_.isEmpty(methods)) {
            methods.forEach((method) => {
                this.angularScript.setMethod(
                    method.getName(),
                    method.getParams(),
                    method.getBody()
                );
            });
        }
    }

    transformConstructor() {
        let method = this.reactScript.getMethod("constructor");
        let body = (method && method.getBody().body) || [];
        this.angularScript.setConstructor(body);
    }
}

class Script {
    executor = null;
    ast = null;

    constructor(ast, options = {}) {
        this.ast = ast;
        traverse(ast, {
            ClassDeclaration: (path) => {
                let classDeclaration = new ClassDeclaration(path);
                let extendsObj = classDeclaration.getExtends();
                if (extendsObj.property.name === "Controller") {
                    this.executor = new Controller(ast);
                } else if (extendsObj.property.name === "Component") {
                    this.executor = new Component(ast, options);
                }
            },
        });
    }

    getCode() {
        let path = this.executor.reactScript.getClass().getPath();
        this.executor.reactScript.removePropType();
        if (path) {
            path.replaceWith(this.executor.angularScript.getAst());
        }
        return ast2code(this.ast);
    }
}

module.exports = Script;
