const ClassMethod = require("../types/ClassMethod");
const { types, template, traverse } = require("@babel/core");
const { findByPattern, ucfrist } = require("../utils");
const _ = require("underscore");
const ClassProperty = require("../types/ClassProperty");
class BaseScript {
    constructor(ast) {
        this.ast = ast;
        // this.transformFunctionParams();
        this.transformRequest();
        this.transformUtils();
        this.transformAssert();
        this.transformComment();
    }

    transformComment() { 
        traverse(this.ast, {
            ClassProperty(path) {
                let classProperty = new ClassProperty(path);
                classProperty.getNode().trailingComments = null;
            },
            ClassMethod(path) {
                let classMethod = new ClassMethod(path);
                classMethod.getNode().trailingComments = null;
            },
        });
    }

    transformAssert() {
        traverse(this.ast, {
            ClassMethod(path) {
                let classMethod = new ClassMethod(path);
                let comment = classMethod.getLeadingComments();
                let params = classMethod.getParams();
                if (_.isEmpty(params) || _.isEmpty(comment)) {
                    return;
                }
                params.forEach(function (item) {
                    let pattern = new RegExp(`\@param {(.*?)} ${item.name}`);
                    let match = comment.value.match(pattern);
                    if (_.isArray(match) && match[1]) {
                        classMethod.prepend([
                            template(
                                `
                                if (!\_.is${ucfrist(match[1])}(${item.name})) {
                                    throw new Error("The type of the ${
                                        item.name
                                    } parameter of the ${classMethod.getName()} method must be ${
                                    match[1]
                                }.");
                                }
                            `,
                                {
                                    placeholderPattern: /^[$A-Z0-9]+$/,
                                }
                            )(),
                        ]);
                    }
                });
            },
        });
    }

    transformUtils() {
        traverse(this.ast, {
            Identifier(path) {
                if (
                    [
                        "isEmpty",
                        "isString",
                        "isArray",
                        "isNumber",
                        "isBoolean",
                    ].includes(path.node.name)
                ) {
                    let parent = path.findParent((path) =>
                        path.isCallExpression()
                    );
                    if (parent && types.isIdentifier(parent.node.callee)) {
                        parent.replaceWith(
                            template(`_.${path.node.name}(%%value%%)`)({
                                value: parent.node.arguments[0],
                            })
                        );
                    }
                }
            },
        });
    }

    // transformFunctionParams() {
    //     traverse(this.ast, {
    //         ClassMethod(path) {
    //             let classMethod = new ClassMethod(path);
    //             let values = [];
    //             if (classMethod.getParams().length === 0) {
    //                 return;
    //             }
    //             classMethod.getParams().forEach(function (item) {
    //                 item.typeAnnotation = null;
    //                 values.push(item.name);
    //             });

    //             classMethod.prepend([
    //                 template(`
    //                     [${values.join(",")}].forEach(function(value, key){
    //                         if([undefined, null, ""].includes(value)) {
    //                             throw new Error(\`\${["${values.join(
    //                                 '","'
    //                             )}"][key]} parameter of ${
    //                     classMethod.getName().name
    //                 } method is required.\`)
    //                         }
    //                     })
    //                 `)(),
    //             ]);
    //         },
    //     });
    // }

    /**
     * 转换请求方法
     * @example
     * // 编译前：
     * request.post("/path/to/backend", {foo: "bar"}).then(function() {
     *      // doSomething...
     * })
     * // 编译后：
     * $.ajax({
     *      url: "/path/to/backend",
     *      method: "POST",
     *      contentType: "application/json",
     *      dataType: "json",
     *      data: JSON.stringify({foo: "bar"}),
     *      success: function() {
     *          // doSomething...
     *      }
     * })
     */
    transformRequest() {
        traverse(this.ast, {
            ExpressionStatement(path) {
                if (findByPattern("request.post", path.node)) {
                    let params = path.node.expression.callee.object.arguments;
                    let func = path.node.expression.arguments[0];
                    let data = "";
                    let placeholder = {
                        url: params[0],
                        success: func,
                    };
                    if (params[1] && params[1].properties.length > 0) {
                        data = "data: JSON.stringify(%%data%%),";
                        placeholder["data"] = params[1] || {};
                    }
                    path.replaceWith(
                        template(
                            `$.ajax({
                            url: %%url%% ,
                            method: "POST",
                            contentType: "application/json",
                            dataType: "json",
                            ${data}
                            success: %%success%%
                        })`
                        )(placeholder)
                    );
                }
            },
        });
    }
}

module.exports = BaseScript;
