const { traverse, types } = require("@babel/core");
const ClassProperty = require("../types/ClassProperty");
const ClassMethod = require("../types/ClassMethod");
const ObjectExpression = require("../types/ObjectExpression");
const ClassDeclaration = require("../types/ClassDeclaration");
const ImportDeclaration = require("../types/ImportDeclaration");
const MemberExpression = require("../types/MemberExpression");
const Path = require("../types/Path");
const { findPath } = require("../utils");

class Method extends Path {
    constructor(path) {
        super(path);
    }

    getReturnValue() {
        return this.node.body.body[0].argument;
    }
}

class Script {
    constructor(ast) {
        this.ast = ast;
    }

    setState() {
        traverse(this.ast, {
            Identifier: (path) => {
                if (path.node.name === "setState") {
                    let parent = path.findParent((path) =>
                        path.isExpressionStatement()
                    );
                    if (parent) {
                        let [key, value] = parent.node.expression.arguments;
                        if (typeof key.value !== "string") {
                            return;
                        } 
                        parent.node.expression.arguments[0] = types.objectExpression(
                            [
                                types.objectProperty(
                                    types.identifier(key.value),
                                    value
                                ),
                            ]
                        );
                        parent.node.expression.arguments.splice(1, 1);
                    }
                }
            },
        });
    }

    setExtends() {
        traverse(this.ast, {
            ClassDeclaration(path) {
                let classDeclaration = new ClassDeclaration(path);
                classDeclaration.setExtends(
                    types.memberExpression(
                        types.identifier("React"),
                        types.identifier("Component")
                    )
                );
            },
        });
    }

    removePropType() {
        traverse(this.ast, {
            Identifier(path) {
                if (path.node.name === "propTypes") {
                    let parent = path.findParent((p) =>
                        p.isExpressionStatement()
                    );
                    if (parent) {
                        parent.remove();
                    }
                }
            },
        });
    }

    getClass() {
        let classPath;
        traverse(this.ast, {
            ClassDeclaration(path) {
                classPath = new ClassDeclaration(path);
            },
        });
        return classPath;
    }

    getIdentifier(name) {
        let identifier;
        traverse(this.ast, {
            Identifier(path) {
                if (path.node.name === name) {
                    identifier = path;
                }
            },
        });
        return identifier;
    }

    getComponentDidUpdate() {
        return this.getMethod("componentDidUpdate");
    }

    getImport() {
        let paths = [];
        traverse(this.ast, {
            ImportDeclaration(path) {
                paths.push(new ImportDeclaration(path));
            },
        });
        return paths;
    }

    getClassName() {
        let className;
        traverse(this.ast, {
            ClassDeclaration(path) {
                className = new ClassDeclaration(path);
            },
        });
        return className;
    }

    getDefaultProps() {
        let props;
        traverse(this.ast, {
            Identifier(path) {
                if (types.isIdentifier(path.node, { name: "defaultProps" })) {
                    let parent = path.findParent((p) =>
                        types.isAssignmentExpression(p)
                    );
                    if (parent) {
                        props = new ObjectExpression(
                            parent.container.expression.right
                        );
                    }
                }
            },
        });
        return props;
    }

    getProps() {
        let props;
        traverse(this.ast, {
            Identifier(path) {
                if (types.isIdentifier(path.node, { name: "propTypes" })) {
                    let parent = path.findParent((p) =>
                        types.isAssignmentExpression(p)
                    );
                    if (parent) {
                        props = new ObjectExpression(
                            parent.container.expression.right
                        );
                    }
                }
            },
        });
        return props;
    }

    getProperty(name) {
        let property;
        traverse(this.ast, {
            ClassProperty(path) {
                if (types.isIdentifier(path.node.key, { name: name })) {
                    property = new ClassProperty(path);
                }
            },
        });
        return property;
    }

    removeProperty(name) {
        traverse(this.ast, {
            ClassProperty(path) {
                if (types.isIdentifier(path.node.key, { name: name })) {
                    path.remove();
                }
            },
        });
    }

    getProperties() {
        let properties = [];
        traverse(this.ast, {
            ClassProperty(path) {
                if (!types.isIdentifier(path.node.key, { name: "state" })) {
                    properties.push(new ClassProperty(path));
                }
            },
        });
        return properties;
    }

    getMethod(name) {
        let method;
        traverse(this.ast, {
            ClassMethod(path) {
                if (types.isIdentifier(path.node.key, { name: name })) {
                    method = new ClassMethod(path);
                }
            },
        });
        return method;
    }

    getMethods() {
        let methods = [];
        traverse(this.ast, {
            ClassMethod(path) {
                if (
                    !types.isIdentifier(path.node.key, { name: "constructor" })
                ) {
                    methods.push(new ClassMethod(path));
                }
            },
        });
        return methods;
    }

    getRender() {
        let render;
        traverse(this.ast, {
            ClassMethod: (path) => {
                if (path.node.key.name === "render") {
                    render = new Method(path);
                }
            },
        });
        return render;
    }
}

module.exports = Script;
