const { traverse, types } = require("@babel/core");
const Path = require("../types/Path");
const { tpl2code } = require("../../lib/utils");

class InnerHTML {
    constructor(path) {
        this.path = path;
        this.node = this.path.node;
    }
    getValue(name) {
        let attr = this.node.value.expression.properties.find(function (item) {
            return item.key.name === name;
        });
        return attr.value;
    }
}

class ClassName extends Path {
    constructor(path) {
        super(path);
    }

    getValue() {
        return this.node.value;
    }
}

class Attribute extends Path {
    constructor(path) {
        super(path);
    }

    getValue() {
        return this.node.value;
    }

    getName() {
        return this.node.name.name;
    }

    getElementName() {
        return this.path.findParent((item) => item.isJSXElement()).node
            .openingElement.name.name;
    }
}

class Children extends Path {
    constructor(path) {
        super(path);
    }
}

class Loop extends Path {
    constructor(path) {
        super(path);
    }

    getElementName() {
        return this.node.expression.arguments[0].body.body[0].argument
            .openingElement.name;
    }

    getItem() {
        return this.node.expression.arguments[0].params[0].name;
    }

    getItems() {
        return tpl2code(this.node.expression.callee.object);
    }

    getAttributes() {
        return this.node.expression.arguments[0].body.body[0].argument
            .openingElement.attributes;
    }

    getChildren() {
        return this.node.expression.arguments[0].body.body[0].argument.children;
    }

    getKey() {
        if (this.node.expression.arguments[0].params.length === 2) {
            return this.node.expression.arguments[0].params[1].name;
        } else {
            return "";
        }
    }
}

class If extends Path {
    constructor(path) {
        super(path);
    }

    getCondition() {
        return this.node.expression.left;
    }

    getElementName() {
        return this.node.expression.right.openingElement.name.name;
    }

    getAttributes() {
        return this.node.expression.right.openingElement.attributes;
    }

    getChildren() {
        return this.node.expression.right.children;
    }
}

class Event extends Path {
    constructor(path) {
        super(path);
    }

    getEventName() {
        return this.node.name.name;
    }

    getElementName() {
        return this.path.findParent((item) => item.isJSXElement()).node
            .openingElement.name.name;
    }

    getFunctionName() {
        if (types.isMemberExpression(this.node.value.expression)) {
            return this.node.value.expression.property.name;
        } else if (types.isCallExpression(this.node.value.expression)) {
            return this.node.value.expression.callee.object.property.name;
        }
    }

    getFunctionParams() {
        return this.node.value.expression.arguments;
    }
}

class Variable extends Path {
    constructor(path) {
        super(path);
    }

    getName() {
        return tpl2code(this.node.expression);
    }
}

class Style extends Path {
    constructor(path) {
        super(path);
    }

    getValue() {
        return this.node.value;
    }
}

class Template {
    attributeMaps = ["checked", "disabled", "src", "value"];

    constructor(ast) {
        this.ast = ast;
    }

    getStyles() {
        let paths = [];
        traverse(this.ast, {
            JSXAttribute: (path) => {
                if (
                    types.isJSXExpressionContainer(path.node.value) &&
                    path.node.name.name === "style"
                ) {
                    paths.push(new Style(path));
                }
            },
        });
        return paths;
    }

    getVars() {
        let paths = [];
        traverse(this.ast, {
            JSXExpressionContainer(path) {
                let parent = path.findParent((path) => path.isJSXElement());
                if (parent) {
                    paths.push(new Variable(path));
                }
            },
        });
        return paths;
    }

    getIfs() {
        let paths = [];
        traverse(this.ast, {
            JSXExpressionContainer(path) {
                if (types.isLogicalExpression(path.node.expression)) {
                    paths.push(new If(path));
                }
            },
        });
        return paths;
    }

    getChildren() {
        let children;
        traverse(this.ast, {
            JSXExpressionContainer(path) {
                if (
                    types.isIdentifier(path.node.expression, {
                        name: "children",
                    })
                ) {
                    children = new Children(path);
                }
            },
        });
        return children;
    }

    getAttributes() {
        let attributes = [];
        traverse(this.ast, {
            JSXAttribute: (path) => {
                if (
                    types.isJSXExpressionContainer(path.node.value)
                    // &&
                    // this.attributeMaps.includes(path.node.name.name)
                ) {
                    attributes.push(new Attribute(path));
                }
            },
        });
        return attributes;
    }

    getLoops() {
        let paths = [];
        traverse(this.ast, {
            JSXExpressionContainer(path) {
                if (
                    types.isCallExpression(path.node.expression) &&
                    types.isMemberExpression(path.node.expression.callee) &&
                    path.node.expression.callee.property.name === "map"
                ) {
                    paths.push(new Loop(path));
                }
            },
        });
        return paths;
    }

    getExpressionContainer() {
        let containers = [];
        traverse(this.ast, {
            JSXExpressionContainer(path) {
                containers.push(path);
            },
        });
        return containers;
    }

    getClassNames() {
        let className = [];
        traverse(this.ast, {
            JSXAttribute(path) {
                if (
                    types.isJSXIdentifier(path.node.name, { name: "className" })
                ) {
                    className.push(new ClassName(path));
                }
            },
        });
        return className;
    }

    getEvents() {
        let paths = [];
        traverse(this.ast, {
            JSXAttribute(path) {
                if (
                    types.isJSXExpressionContainer(path.node.value) &&
                    ((types.isMemberExpression(path.node.value.expression) &&
                        types.isThisExpression(
                            path.node.value.expression.object
                        )) ||
                        (types.isCallExpression(path.node.value.expression) &&
                            types.isThisExpression(
                                path.node.value.expression.callee.object.object
                            )))
                ) {
                    paths.push(new Event(path));
                }
                //if (path.node.name.name.indexOf("on") === 0) {
                // let parent = path.findParent((path) => path.isJSXElement());
                // if (path.node.name.name === "onChange" && parent.node.openingElement.name.name !== "input") {
                //     return;
                // }
                //  paths.push(new Event(path));
                //}
            },
        });
        return paths;
    }

    getInnerHTML() {
        let paths = [];
        traverse(this.ast, {
            JSXAttribute(path) {
                if (path.node.name.name === "dangerouslySetInnerHTML") {
                    paths.push(new InnerHTML(path));
                }
            },
        });
        return paths;
    }
}

module.exports = Template;
