const t = require("@babel/types");
const template = require("@babel/template").default;
const traverse = require("@babel/traverse").default;
const generate = require("@babel/generator").default;
const { parse } = require("@babel/parser");
const fs = require("fs");
const prettier = require("prettier");

/**
 * 查找路径
 * @param {string} type
 * @param {function} filter
 */
function findPath(ast, type, filter) {
    let path;
    let traverses = {};
    traverses[type] = function (p) {
        path = filter(p);
    };
    traverse(ast, traverses);
    return path;
}

function ucfrist(name) {
    return name.charAt(0).toUpperCase() + name.slice(1);
}

function findByPattern(pattern, ast) {
    return new RegExp(pattern).test(ast2code(ast));
}

function getAst(content, type) {
    if (type === "script") {
        return parse(content, {
            sourceType: "module",
            plugins: ["classProperties", "typescript"],
        });
    }
}

function getFileAst(filePath, type = "script") {
    let content = fs.readFileSync(filePath, { encoding: "utf-8" });
    if (type === "script") {
        return parse(content, {
            sourceType: "script",
            plugins: ["classProperties", "typescript"],
        });
    }
}

function writeFileAst(filePath, ast, type = "script", format = false) {
    if (type === "script") {
        fs.writeFileSync(filePath, ast2code(ast, format), {
            encoding: "utf-8",
        });
    }
}

function getJsxAst(code) {
    return parse(code, {
        plugins: ["jsx"],
    });
}

function tpl2code(ast) {
    return generate(ast, {
        minified: false,
    }).code.replace(/;$/, "");
}

function ast2code(ast, format = false) {
    if (format) {
        return prettier.format(generate(ast).code, {
            parser: "babel",
            tabWidth: 4,
        });
    } else {
        return generate(ast, {}).code;
    }
}

function hasWatch(ast) {
    let bool = false;
    traverse(ast, {
        ClassProperty(path) {
            if (path.node && t.isIdentifier(path.node.key, { name: "watch" })) {
                bool = true;
            }
        },
    });
    return bool;
}

function hasConstructor(ast) {
    let bool = false;
    traverse(ast, {
        ClassMethod(path) {
            if (path.node.key.name === "constructor") {
                return true;
            }
        },
    });
    return bool;
}

function isChildren(ast) {
    let bool = false;
    traverse(ast, {
        JSXExpressionContainer(path) {
            if (t.isIdentifier(path.node.expression, { name: "children" })) {
                return true;
            }
        },
    });
    return bool;
}

function inArray4Identifier(path, array, callback) {
    array.forEach((element) => {
        if (t.isIdentifier(path.node, { name: element })) {
            callback(element);
        }
    });
}

function toCamelCase(name) {
    return name.replace(name[0], name[0].toLowerCase());
}

function toKebabCase(name) {
    return name.replace(/([A-Z])/g, "-$1").toLowerCase();
}

/**
 * 切换单花括号至双花括号
 * @param {Object} path 节点路径
 */
function toDoubleBracket(path) {
    if (!t.isJSXExpressionContainer()) {
        return;
    }
    path.replaceWith(
        t.stringLiteral(
            "{" + generate(path.node, { minified: true }).code + "}"
        )
    );
}

function toSetterMethod(path, method) {
    if (!t.isAssignmentExpression(path)) {
        return;
    }
    let node = path.node;
    path.replaceWith(
        template(`this.${method}(OPTIONS)`)({
            OPTIONS: t.objectExpression([
                t.objectProperty(node.left.property, node.right),
            ]),
        })
    );
}

function replaceElement(path, element) {
    if (t.isJSXIdentifier(path.node.openingElement.name, { name: "slot" })) {
        let parent = path.findParent((path) => path.isJSXElement());
        parent.node.openingElement.attributes.push(
            t.JSXAttribute(t.JSXIdentifier("ng-transclude"), null)
        );
        path.remove();
    }
}

function addPrefix(path, attributes, handle) {
    attributes.forEach((element) => {
        if (t.isJSXIdentifier(path.node, { name: element })) {
            path.node.name = handle(element);
        }
    });
}

function printNodeKey(node, list = []) {
    let keys = [];
    for (const key in node) {
        if (Object.hasOwnProperty.call(node, key)) {
            keys.push(key);
        }
    }
    console.log(keys);
}

function printNode(node, list = []) {
    let properties = {};
    for (const key in node) {
        if (Object.hasOwnProperty.call(node, key)) {
            const element = node[key];
            if (list.findIndex((item) => key === item) !== -1) {
                properties[key] = element;
            }
        }
    }
    console.log(properties);
}

module.exports.toCamelCase = toCamelCase;
module.exports.addPrefix = addPrefix;
module.exports.printNode = printNode;
module.exports.printNodeKey = printNodeKey;
module.exports.toSetterMethod = toSetterMethod;
module.exports.toDoubleBracket = toDoubleBracket;
module.exports.inArray4Identifier = inArray4Identifier;
module.exports.isChildren = isChildren;
module.exports.hasConstructor = hasConstructor;
module.exports.hasWatch = hasWatch;
module.exports.ast2code = ast2code;
module.exports.tpl2code = tpl2code;
module.exports.getAst = getAst;
module.exports.getJsxAst = getJsxAst;
module.exports.toKebabCase = toKebabCase;
module.exports.findByPattern = findByPattern;
module.exports.findPath = findPath;
module.exports.getFileAst = getFileAst;
module.exports.writeFileAst = writeFileAst;
module.exports.ucfrist = ucfrist  ;
